from unittest import TestCase, mock
from unittest.mock import patch, mock_open

from svwebscraper.exporter import SchoolCSVExporter


class School:
    def __init__(self, name, address, postal_code, city, phone_number):
        self.name = name
        self.address = address
        self.postal_code = postal_code
        self.city = city
        self.phone_number = phone_number

    def get_name(self):
        return self.name

    def get_address(self):
        return self.address

    def get_postal_code(self):
        return self.postal_code

    def get_city(self):
        return self.city

    def get_phone_number(self):
        return self.phone_number


class ExporterTest(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.schools = [
            School('Freie Waldorfschule Wien-West', 'Seuttergasse 29', '1120', 'Wien', '01/1234567'),
            School('Mittelschule des Schulvereins der Dominikanerinnen', 'Schlossberggasse 17', '1130', 'Wien',
                   '01/1234567'),
            School('Neue Mittelschule Wien', 'Neubaugasse 42', '1070', 'Wien', '01/1234567')
        ]

    def test_export_to_file(self):
        with patch('builtins.open', mock_open()) as file_handle:
            exporter = SchoolCSVExporter(ExporterTest.schools)
            exporter.export_to_file('testfile.csv')

            self.assertEqual(file_handle.return_value.write.call_count, 4)

            file_handle.return_value.write.assert_has_calls([
                mock.call('NAME;ADRESSE;PLZ;ORT;TEL_NR\n'),
                mock.call('Freie Waldorfschule Wien-West;Seuttergasse 29;1120;Wien;01/1234567\n'),
                mock.call(
                    'Mittelschule des Schulvereins der Dominikanerinnen;Schlossberggasse 17;1130;Wien;01/1234567\n'),
                mock.call('Neue Mittelschule Wien;Neubaugasse 42;1070;Wien;01/1234567\n')
            ])

    def test_export_to_file_with_custom_delimiters(self):
        with patch('builtins.open', mock_open()) as file_handle:
            exporter = SchoolCSVExporter(ExporterTest.schools, delimiter=',')
            exporter.export_to_file('testfile.csv')

            self.assertEqual(file_handle.return_value.write.call_count, 4)

            file_handle.return_value.write.assert_has_calls([
                mock.call('NAME,ADRESSE,PLZ,ORT,TEL_NR\n'),
                mock.call('Freie Waldorfschule Wien-West,Seuttergasse 29,1120,Wien,01/1234567\n'),
                mock.call(
                    'Mittelschule des Schulvereins der Dominikanerinnen,Schlossberggasse 17,1130,Wien,01/1234567\n'),
                mock.call('Neue Mittelschule Wien,Neubaugasse 42,1070,Wien,01/1234567\n')
            ])

    def test_export_to_file_with_custom_newline(self):
        with patch('builtins.open', mock_open()) as file_handle:
            exporter = SchoolCSVExporter(ExporterTest.schools, newline='\r\n')
            exporter.export_to_file('testfile.csv')

            self.assertEqual(file_handle.return_value.write.call_count, 4)

            file_handle.return_value.write.assert_has_calls([
                mock.call('NAME;ADRESSE;PLZ;ORT;TEL_NR\r\n'),
                mock.call('Freie Waldorfschule Wien-West;Seuttergasse 29;1120;Wien;01/1234567\r\n'),
                mock.call(
                    'Mittelschule des Schulvereins der Dominikanerinnen;Schlossberggasse 17;1130;Wien;01/1234567\r\n'),
                mock.call('Neue Mittelschule Wien;Neubaugasse 42;1070;Wien;01/1234567\r\n')
            ])


import os

HEADER = ['NAME', 'ADRESSE', 'PLZ', 'ORT', 'TEL_NR']
EXPORT_DIRECTORY = './export/'


class SchoolCSVExporter:
    def __init__(self, schools, delimiter=';', newline='\n'):
        self.schools = schools
        self.delimiter = delimiter
        self.newline = newline

        if not os.path.isdir(EXPORT_DIRECTORY):
            os.makedirs(EXPORT_DIRECTORY)

    def export_to_file(self, filename, append=False):
        print(f'Export {len(self.schools)} schools to csv file "{filename}"')

        if append:
            mode = 'w+'
        else:
            mode = 'w'

        with open(EXPORT_DIRECTORY + filename, mode) as file:
            self._append_header_to_file(file)
            for school in self.schools:
                self._append_school_to_file(file, school)

    def _append_header_to_file(self, file):
        file.write(self.delimiter.join(HEADER) + self.newline)

    def _append_school_to_file(self, file, school):
        school_fields = [
            school.get_name(),
            school.get_address(),
            school.get_postal_code(),
            school.get_city(),
            school.get_phone_number()
        ]
        file.write(self.delimiter.join(school_fields) + self.newline)



class RequestFailedException(Exception):
    def __init__(self, status_code):
        message = f'request failed with status code {status_code}.'
        super(RequestFailedException, self).__init__(message)



import click

from urllib.parse import urljoin
from svwebscraper.exporter import SchoolCSVExporter
from svwebscraper.webscraper import SchoolDataWebScrapper

DEFAULT_BASE_URL = 'http://www.schulverzeichnis.eu/typ/'
SCHOOL_TYPES = ['neue-mittelschule', 'ahs-mit-nms', 'hauptschule', 'sonderschule']
DEFAULT_SCHOOL_TYPE = SCHOOL_TYPES[0]
DEFAULT_QUERY = '?bundesland=wien'


@click.command()
@click.option('--csv', nargs=1, type=click.File())
@click.option('--baseurl', nargs=1, default=DEFAULT_BASE_URL)
@click.option('--type', nargs=1, default=DEFAULT_SCHOOL_TYPE, type=click.Choice(SCHOOL_TYPES))
@click.option('--query', nargs=1, default=DEFAULT_QUERY)
def scrape_schools(csv, baseurl, type, query):
    url = urljoin(baseurl, type)
    web_scrapper = SchoolDataWebScrapper(f'{url}/{query}')
    schools = web_scrapper.retrieve_school_list()

    if csv:
        exporter = SchoolCSVExporter(schools)
        exporter.export_to_file(csv)



import logging
import requests
from bs4 import BeautifulSoup, Tag

from svwebscraper.exceptions import RequestFailedException


class SchoolDataWebScrapper:
    def __init__(self, url):
        splitted_url = url.split('?')

        self.base_url = url.split('?')[0]

        if len(splitted_url) > 1:
            self.initial_query_part = f"?{''.join(splitted_url[1:])}"
        else:
            self.initial_query_part = ''

    def retrieve_school_list(self):
        print("Start scrapping schools from base url...")

        parsed_web_page = self._parse_web_page(self.initial_query_part)
        schools = parsed_web_page.get_schools()

        print(f'{len(schools)} schools scrapped from "{self.base_url}"')

        while parsed_web_page.has_next():
            parsed_web_page = self._parse_web_page(parsed_web_page.get_next_query_params())
            schools += parsed_web_page.get_schools()

            print(f'{len(schools)} schools scrapped from "{self.base_url}"')

        return schools

    def _parse_web_page(self, query_params):
        html_source = self._fetch_html_source(query_params)
        return ParsedPage(html_source)

    def _fetch_html_source(self, query_params):
        response = requests.get(f"{self.base_url}{query_params}")

        if response.status_code != 200:
            raise RequestFailedException(response.status_code)

        response.encoding = response.apparent_encoding
        return response.text


class ParsedPage:
    def __init__(self, source):
        self.soup = BeautifulSoup(source, 'html.parser')

    def has_next(self):
        return self.get_next_query_params() is not None

    def get_schools(self):
        school_divs = self.soup.find_all('div', class_='school_name')
        schools = list()

        for school_tag in school_divs:
            name_tag = school_tag.find_all('a')[0]
            try:
                schools.append(School(name_tag['href']))
            except RequestFailedException:
                logging.error(f'Parsing school {name_tag["href"]} failed.')

        return schools

    def get_next_query_params(self):
        paging_div = self.soup.find_all('div', class_='paging')[0]

        for child in paging_div.contents:
            if type(child) is Tag and child.text.startswith('weiter'):
                return child['href']
        return None


class School:
    def __init__(self, school_meta_url):
        response = requests.get(school_meta_url)
        response.encoding = response.apparent_encoding

        if response.status_code != 200:
            logging.error(f'Response of request to {school_meta_url} resulted in status_code {response.status_code}.')
            raise RequestFailedException(response.status_code)

        soup = BeautifulSoup(response.text, 'html.parser')
        self.info_div = soup.find_all('p', class_='map_title')[0].parent

    def get_name(self):
        tag = self.info_div.findChild('p', class_='map_title')
        if tag:
            return tag.text
        return None

    def get_address(self):
        tag = self.info_div.findChild('span', itemprop="streetAddress")

        if tag:
            return tag.text
        return None

    def get_postal_code(self):
        tag = self.info_div.findChild('span', itemprop="postalCode")
        if tag:
            return tag.text
        return None

    def get_city(self):
        tag = self.info_div.findChild('span', itemprop="addressLocality")
        if tag:
            return tag.text
        return None

    def get_phone_number(self):
        tag = self.info_div.findChild('span', itemprop="telephone")
        if tag:
            return tag.text
        return None



from unittest import TestCase
from unittest.mock import patch

from svwebscraper.webscraper import SchoolDataWebScrapper, RequestFailedException, School, ParsedPage
from tests.util.testdata_loader import load_file_to_string

SCHOOL_LIST_HTML_PAGE_WITH_SUCCESSOR = load_file_to_string('testdata/sv_list_with_succeeding_pages.html')
SCHOOL_LIST_HTML_PAGE_WITH_NO_SUCCESSOR = load_file_to_string('testdata/sv_list_with_no_successor.html')
SCHOOL_DETAIL_HTML_PAGE = load_file_to_string('./testdata/sv_detail_view.html')


class HttpResponseMock:
    def __init__(self, text, status_code, apparent_encoding):
        self.text = text
        self.status_code = status_code
        self.apparent_encoding = apparent_encoding


class SchoolPageTest(TestCase):
    @patch('requests.get', side_effect=lambda _: HttpResponseMock(SCHOOL_DETAIL_HTML_PAGE, 200, 'UTF-8'))
    def test_successful_parsing(self, get):
        school = School('')

        self.assertEqual(school.get_name(), 'Neue Mittelschule Wien')
        self.assertEqual(school.get_address(), 'Neubaugasse 42')
        self.assertEqual(school.get_postal_code(), '1070')
        self.assertEqual(school.get_city(), 'Wien')
        self.assertEqual(school.get_phone_number(), '01/526 15 94')


class ParsedPageTest(TestCase):
    def test_parse_page_with_successor(self):
        parsed_page = ParsedPage(SCHOOL_LIST_HTML_PAGE_WITH_SUCCESSOR)

        self.assertTrue(parsed_page.has_next())
        self.assertEqual(parsed_page.get_next_query_params(), '?bundesland=wien&start=20')
        self.assertEqual(len(parsed_page.get_schools()), 20)

    def test_parse_page_without_successor(self):
        parsed_page = ParsedPage(SCHOOL_LIST_HTML_PAGE_WITH_NO_SUCCESSOR)

        self.assertFalse(parsed_page.has_next())
        self.assertIsNone(parsed_page.get_next_query_params())
        self.assertEqual(len(parsed_page.get_schools()), 8)


class SchoolDataWebScrapperTest(TestCase):
    @patch('requests.get', side_effect=[HttpResponseMock(SCHOOL_LIST_HTML_PAGE_WITH_SUCCESSOR, 200, 'UTF-8'),
                                        HttpResponseMock(SCHOOL_LIST_HTML_PAGE_WITH_NO_SUCCESSOR, 200, 'UTF-8')])
    @patch('svwebscraper.webscraper.School')
    def test_retrieve_schools(self, school_mock, get_mock):
        webscraper = SchoolDataWebScrapper("https://schulverzeichnis.eu")
        schools = webscraper.retrieve_school_list()

        self.assertEqual(len(schools), 28)

    @patch('requests.get', side_effect=[HttpResponseMock(SCHOOL_LIST_HTML_PAGE_WITH_SUCCESSOR, 200, 'UTF-8')])
    def test_successful_fetch_html_source(self, get):
        webscraper = SchoolDataWebScrapper("https://schulverzeichnis.eu")
        retrieved_html = webscraper._fetch_html_source("?bundesland=wien")

        self.assertEqual(retrieved_html, SCHOOL_LIST_HTML_PAGE_WITH_SUCCESSOR)

    @patch('requests.get', side_effect=[HttpResponseMock(None, 404, 'UTF-8')])
    def test_fetch_html_source_with_unsuccessful_statuscode(self, get):
        webscraper = SchoolDataWebScrapper("https://schulverzeichnis.eu")

        with self.assertRaises(RequestFailedException):
            webscraper._fetch_html_source("?bundesland=wien")
